@@ -548,7 +548,7 @@ close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 		entry = NULL;
 	} else if ((soup_message_headers_get_encoding (entry->headers) == SOUP_ENCODING_CHUNKED) ||
 		   entry->length != (gsize) content_length) {
-		/** Two options here:
+		/* Two options here:
 		 *
 		 * 1. "chunked" data, entry was temporarily added to
 		 * cache (as content-length is 0) and now that we have
@@ -560,7 +560,7 @@ close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 		 * transmission (typically compressed) and thus we
 		 * have to substract the content-length value that was
 		 * added to the cache and add the unencoded length
-		 **/
+		 */
 		gint length_to_add = entry->length - content_length;
 
 		/* Make room in cache if needed */
@@ -757,7 +757,7 @@ lru_compare_func (gconstpointer a, gconstpointer b)
 	SoupCacheEntry *entry_a = (SoupCacheEntry *)a;
 	SoupCacheEntry *entry_b = (SoupCacheEntry *)b;
 
-	/** The rationale of this sorting func is
+	/* The rationale of this sorting func is
 	 *
 	 * 1. sort by hits -> LRU algorithm, then
 	 *
@@ -766,7 +766,7 @@ lru_compare_func (gconstpointer a, gconstpointer b)
 	 *
 	 * 3. sort by size, replace first small size resources as they
 	 * are cheaper to download
-	 **/
+	 */
 
 	/* Sort by hits */
 	if (entry_a->hits != entry_b->hits)
@@ -1258,8 +1258,8 @@ soup_cache_class_init (SoupCacheClass *cache_class)
  *
  * Returns: a new #SoupCache
  *
- * Since: 2.28
- **/
+ * Since: 2.34
+ */
 SoupCache *
 soup_cache_new (const char *cache_dir, SoupCacheType cache_type)
 {
@@ -1279,7 +1279,9 @@ soup_cache_new (const char *cache_dir, SoupCacheType cache_type)
  * and the cached reply and the time ellapsed since it was cached.
  *
  * Returns: whether or not the @cache has a valid response for @msg
- **/
+ *
+ * Since: 2.34
+ */
 SoupCacheResponse
 soup_cache_has_response (SoupCache *cache, SoupMessage *msg)
 {
@@ -1430,7 +1432,9 @@ soup_cache_has_response (SoupCache *cache, SoupMessage *msg)
  * Calculates whether the @msg can be cached or not.
  *
  * Returns: a #SoupCacheability value indicating whether the @msg can be cached or not.
- **/
+ *
+ * Since: 2.34
+ */
 SoupCacheability
 soup_cache_get_cacheability (SoupCache *cache, SoupMessage *msg)
 {
@@ -1452,19 +1456,19 @@ force_flush_timeout (gpointer data)
 /**
  * soup_cache_flush:
  * @cache: a #SoupCache
- * @session: the #SoupSession associated with the @cache
  *
  * This function will force all pending writes in the @cache to be
  * committed to disk. For doing so it will iterate the #GMainContext
- * associated with the @session (which can be the default one) as long
- * as needed.
- **/
+ * associated with @cache's session as long as needed.
+ *
+ * Since: 2.34
+ */
 void
 soup_cache_flush (SoupCache *cache)
 {
 	GMainContext *async_context;
 	SoupSession *session;
-	guint timeout_id;
+	GSource *timeout;
 	gboolean forced = FALSE;
 
 	g_return_if_fail (SOUP_IS_CACHE (cache));
@@ -1474,13 +1478,13 @@ soup_cache_flush (SoupCache *cache)
 	async_context = soup_session_get_async_context (session);
 
 	/* We give cache 10 secs to finish */
-	timeout_id = g_timeout_add (10000, force_flush_timeout, &forced);
+	timeout = soup_add_timeout (async_context, 10000, force_flush_timeout, &forced);
 
 	while (!forced && cache->priv->n_pending > 0)
 		g_main_context_iteration (async_context, FALSE);
 
 	if (!forced)
-		g_source_remove (timeout_id);
+		g_source_destroy (timeout);
 	else
 		g_warning ("Cache flush finished despite %d pending requests", cache->priv->n_pending);
 }
@@ -1502,7 +1506,9 @@ clear_cache_item (gpointer data,
  *
  * Will remove all entries in the @cache plus all the cache files
  * associated with them.
- **/
+ *
+ * Since: 2.34
+ */
 void
 soup_cache_clear (SoupCache *cache)
 {
