@@ -3,10 +3,16 @@
  * Copyright (C) 2008 Red Hat, Inc.
  */
 
-#include "test-utils.h"
+#include "config.h"
 
-static SoupSession *session;
-static SoupURI *base_uri;
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include <glib.h>
+#include <libsoup/soup.h>
+
+#include "test-utils.h"
 
 typedef struct {
 	SoupSession *session;
@@ -15,6 +21,15 @@ typedef struct {
 	gboolean streaming;
 } PutTestData;
 
+static SoupBuffer *
+error_chunk_allocator (SoupMessage *msg, gsize max_len, gpointer user_data)
+{
+	/* This should never be called, because there is no response body. */
+	debug_printf (1, "  error_chunk_allocator called!\n");
+	errors++;
+	return soup_buffer_new (SOUP_MEMORY_TAKE, g_malloc (100), 100);
+}
+
 static void
 write_next_chunk (SoupMessage *msg, gpointer user_data)
 {
@@ -22,9 +37,9 @@ write_next_chunk (SoupMessage *msg, gpointer user_data)
 
 	debug_printf (2, "  writing chunk %d\n", ptd->next);
 
-	if (ptd->streaming && ptd->next > 0) {
-		soup_test_assert (ptd->chunks[ptd->next - 1] == NULL,
-				  "next chunk requested before last one freed");
+	if (ptd->streaming && ptd->next > 0 && ptd->chunks[ptd->next - 1]) {
+		debug_printf (1, "  error: next chunk requested before last one freed!\n");
+		errors++;
 	}
 
 	if (ptd->next < G_N_ELEMENTS (ptd->chunks)) {
@@ -53,8 +68,8 @@ write_next_chunk_streaming_hack (SoupMessage *msg, gpointer user_data)
 		soup_message_body_wrote_chunk (msg->request_body, chunk);
 		soup_buffer_free (chunk);
 	} else {
-		soup_test_assert (chunk,
-				  "written chunk does not exist");
+		debug_printf (1, "  error: written chunk does not exist!\n");
+		errors++;
 	}
 	write_next_chunk (msg, user_data);
 }
@@ -80,8 +95,8 @@ clear_buffer_ptr (gpointer data)
 		g_free ((char *)(*buffer_ptr)->data);
 		*buffer_ptr = NULL;
 	} else {
-		soup_test_assert (*buffer_ptr,
-				  "chunk is already clear");
+		debug_printf (2, "  chunk is already clear!\n");
+		errors++;
 	}
 }
 
@@ -139,20 +154,26 @@ typedef enum {
 } RequestTestFlags;
 
 static void
-do_request_test (gconstpointer data)
+do_request_test (SoupSession *session, SoupURI *base_uri, RequestTestFlags flags)
 {
-	RequestTestFlags flags = GPOINTER_TO_UINT (data);
-	SoupURI *uri;
+	SoupURI *uri = base_uri;
 	PutTestData ptd;
 	SoupMessage *msg;
 	const char *client_md5, *server_md5;
 	GChecksum *check;
 	int i, length;
 
-	if (flags & RESTART)
-		uri = soup_uri_new_with_base (base_uri, "/redirect");
-	else
+	debug_printf (1, "PUT");
+	if (flags & HACKY_STREAMING)
+		debug_printf (1, " w/ hacky streaming");
+	else if (flags & PROPER_STREAMING)
+		debug_printf (1, " w/ proper streaming");
+	if (flags & RESTART) {
+		debug_printf (1, " and restart");
 		uri = soup_uri_copy (base_uri);
+		soup_uri_set_path (uri, "/redirect");
+	}
+	debug_printf (1, "\n");
 
 	ptd.session = session;
 	setup_request_body (&ptd);
@@ -170,6 +191,7 @@ do_request_test (gconstpointer data)
 	msg = soup_message_new_from_uri ("PUT", uri);
 	soup_message_headers_set_encoding (msg->request_headers, SOUP_ENCODING_CHUNKED);
 	soup_message_body_set_accumulate (msg->request_body, FALSE);
+	soup_message_set_chunk_allocator (msg, error_chunk_allocator, NULL, NULL);
 	if (flags & HACKY_STREAMING) {
 		g_signal_connect (msg, "wrote_chunk",
 				  G_CALLBACK (write_next_chunk_streaming_hack), &ptd);
@@ -196,19 +218,35 @@ do_request_test (gconstpointer data)
 			  G_CALLBACK (wrote_body_data), &ptd);
 	soup_session_send_message (session, msg);
 
-	soup_test_assert_message_status (msg, SOUP_STATUS_CREATED);
-	g_assert_null (msg->request_body->data);
-	g_assert_cmpint (msg->request_body->length, ==, length);
-	g_assert_cmpint (length, ==, ptd.nwrote);
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		debug_printf (1, "  message failed: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
+
+	if (msg->request_body->data) {
+		debug_printf (1, "  msg->request_body set!\n");
+		errors++;
+	}
+	if (msg->request_body->length != length || length != ptd.nwrote) {
+		debug_printf (1, "  sent length mismatch: %d vs %d vs %d\n",
+			      (int)msg->request_body->length, length, ptd.nwrote);
+		errors++;
+	}
 
 	server_md5 = soup_message_headers_get_one (msg->response_headers,
 						   "Content-MD5");
-	g_assert_cmpstr (client_md5, ==, server_md5);
+	if (!server_md5 || strcmp (client_md5, server_md5) != 0) {
+		debug_printf (1, "  client/server data mismatch: %s vs %s\n",
+			      client_md5, server_md5 ? server_md5 : "(null)");
+		errors++;
+	}
 
 	g_object_unref (msg);
 	g_checksum_free (check);
 
-	soup_uri_free (uri);
+	if (uri != base_uri)
+		soup_uri_free (uri);
 }
 
 typedef struct {
@@ -224,8 +262,10 @@ chunk_allocator (SoupMessage *msg, gsize max_len, gpointer user_data)
 
 	debug_printf (2, "  allocating chunk\n");
 
-	soup_test_assert (gtd->current_chunk == NULL,
-			  "error: next chunk allocated before last one freed");
+	if (gtd->current_chunk) {
+		debug_printf (1, "  error: next chunk allocated before last one freed!\n");
+		errors++;
+	}
 	gtd->current_chunk = soup_buffer_new_with_owner (g_malloc (6), 6,
 							 &gtd->current_chunk,
 							 clear_buffer_ptr);
@@ -249,40 +289,56 @@ got_chunk (SoupMessage *msg, SoupBuffer *chunk, gpointer user_data)
 }
 
 static void
-do_response_test (void)
+do_response_test (SoupSession *session, SoupURI *base_uri)
 {
 	GetTestData gtd;
 	SoupMessage *msg;
 	const char *client_md5, *server_md5;
 
+	debug_printf (1, "GET\n");
+
 	gtd.current_chunk = NULL;
 	gtd.length = 0;
 	gtd.check = g_checksum_new (G_CHECKSUM_MD5);
 
 	msg = soup_message_new_from_uri ("GET", base_uri);
 	soup_message_body_set_accumulate (msg->response_body, FALSE);
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 	soup_message_set_chunk_allocator (msg, chunk_allocator, &gtd, NULL);
-	G_GNUC_END_IGNORE_DEPRECATIONS;
 	g_signal_connect (msg, "got_chunk",
 			  G_CALLBACK (got_chunk), &gtd);
 	soup_session_send_message (session, msg);
 
-	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
-	g_assert_null (msg->response_body->data);
-	g_assert_cmpint (soup_message_headers_get_content_length (msg->response_headers), ==, gtd.length);
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		debug_printf (1, "  message failed: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
+
+	if (msg->response_body->data) {
+		debug_printf (1, "  msg->response_body set!\n");
+		errors++;
+	}
+	if (soup_message_headers_get_content_length (msg->response_headers) != gtd.length) {
+		debug_printf (1, "  received length mismatch: %d vs %d\n",
+			      (int)soup_message_headers_get_content_length (msg->response_headers), gtd.length);
+		errors++;
+	}
 
 	client_md5 = g_checksum_get_string (gtd.check);
 	server_md5 = soup_message_headers_get_one (msg->response_headers,
 						   "Content-MD5");
-	g_assert_cmpstr (client_md5, ==, server_md5);
+	if (!server_md5 || strcmp (client_md5, server_md5) != 0) {
+		debug_printf (1, "  client/server data mismatch: %s vs %s\n",
+			      client_md5, server_md5 ? server_md5 : "(null)");
+		errors++;
+	}
 
 	g_object_unref (msg);
 	g_checksum_free (gtd.check);
 }
 
 /* Make sure TEMPORARY buffers are handled properly with non-accumulating
- * message bodies.
+ * message bodies. Part of https://bugs.webkit.org/show_bug.cgi?id=18343
  */
 
 static void
@@ -297,25 +353,24 @@ temp_test_wrote_chunk (SoupMessage *msg, gpointer session)
 	 * the I/O to stall since soup-message-io will think it's
 	 * done, but it hasn't written Content-Length bytes yet.
 	 */
-	if (chunk)
-		soup_buffer_free (chunk);
-	else {
-		soup_test_assert (chunk, "Lost second chunk");
+	if (!chunk) {
+		debug_printf (1, "  Lost second chunk!\n");
+		errors++;
 		soup_session_abort (session);
-	}
+	} else
+		soup_buffer_free (chunk);
 
 	g_signal_handlers_disconnect_by_func (msg, temp_test_wrote_chunk, session);
 }
 
 static void
-do_temporary_test (void)
+do_temporary_test (SoupSession *session, SoupURI *base_uri)
 {
 	SoupMessage *msg;
 	char *client_md5;
 	const char *server_md5;
 
-	g_test_bug_base ("https://bugs.webkit.org/");
-	g_test_bug ("18343");
+	debug_printf (1, "PUT w/ temporary buffers\n");
 
 	msg = soup_message_new_from_uri ("PUT", base_uri);
 	soup_message_body_append (msg->request_body, SOUP_MEMORY_TEMPORARY,
@@ -330,60 +385,44 @@ do_temporary_test (void)
 			  G_CALLBACK (temp_test_wrote_chunk), session);
 	soup_session_send_message (session, msg);
 
-	soup_test_assert_message_status (msg, SOUP_STATUS_CREATED);
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		debug_printf (1, "  message failed: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
 
 	server_md5 = soup_message_headers_get_one (msg->response_headers,
 						   "Content-MD5");
-	g_assert_cmpstr (client_md5, ==, server_md5);
+	if (!server_md5 || strcmp (client_md5, server_md5) != 0) {
+		debug_printf (1, "  client/server data mismatch: %s vs %s\n",
+			      client_md5, server_md5 ? server_md5 : "(null)");
+		errors++;
+	}
 
 	g_free (client_md5);
 	g_object_unref (msg);
 }
 
-#define LARGE_CHUNK_SIZE 1000000
-
-typedef struct {
-	SoupBuffer *buf;
-	gsize offset;
-} LargeChunkData;
-
 static void
-large_wrote_body_data (SoupMessage *msg, SoupBuffer *chunk, gpointer user_data)
+do_chunk_tests (SoupURI *base_uri)
 {
-	LargeChunkData *lcd = user_data;
+	SoupSession *session;
 
-	soup_assert_cmpmem (chunk->data, chunk->length,
-			    lcd->buf->data + lcd->offset,
-			    chunk->length);
-	lcd->offset += chunk->length;
-}
-
-static void
-do_large_chunk_test (void)
-{
-	SoupMessage *msg;
-	char *buf_data;
-	int i;
-	LargeChunkData lcd;
-
-	msg = soup_message_new_from_uri ("PUT", base_uri);
-
-	buf_data = g_malloc0 (LARGE_CHUNK_SIZE);
-	for (i = 0; i < LARGE_CHUNK_SIZE; i++)
-		buf_data[i] = i & 0xFF;
-	lcd.buf = soup_buffer_new (SOUP_MEMORY_TAKE, buf_data, LARGE_CHUNK_SIZE);
-	lcd.offset = 0;
-	soup_message_body_append_buffer (msg->request_body, lcd.buf);
-	soup_message_body_set_accumulate (msg->request_body, FALSE);
-
-	g_signal_connect (msg, "wrote_body_data",
-			  G_CALLBACK (large_wrote_body_data), &lcd);
-	soup_session_send_message (session, msg);
-
-	soup_test_assert_message_status (msg, SOUP_STATUS_CREATED);
-
-	soup_buffer_free (lcd.buf);
-	g_object_unref (msg);
+	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
+	do_request_test (session, base_uri, 0);
+	debug_printf (2, "\n\n");
+	do_request_test (session, base_uri, PROPER_STREAMING);
+	debug_printf (2, "\n\n");
+	do_request_test (session, base_uri, PROPER_STREAMING | RESTART);
+	debug_printf (2, "\n\n");
+	do_request_test (session, base_uri, HACKY_STREAMING);
+	debug_printf (2, "\n\n");
+	do_request_test (session, base_uri, HACKY_STREAMING | RESTART);
+	debug_printf (2, "\n\n");
+	do_response_test (session, base_uri);
+	debug_printf (2, "\n\n");
+	do_temporary_test (session, base_uri);
+	soup_test_session_abort_unref (session);
 }
 
 static void
@@ -395,7 +434,9 @@ server_callback (SoupServer *server, SoupMessage *msg,
 	char *md5;
 
 	if (g_str_has_prefix (path, "/redirect")) {
-		soup_message_set_redirect (msg, SOUP_STATUS_FOUND, "/");
+		soup_message_set_status (msg, SOUP_STATUS_FOUND);
+		soup_message_headers_replace (msg->response_headers,
+					      "Location", "/");
 		return;
 	}
 
@@ -428,37 +469,26 @@ main (int argc, char **argv)
 {
 	GMainLoop *loop;
 	SoupServer *server;
-	int ret;
+	guint port;
+	SoupURI *base_uri;
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
+	server = soup_test_server_new (TRUE);
 	soup_server_add_handler (server, NULL,
 				 server_callback, NULL, NULL);
+	port = 	soup_server_get_port (server);
 
 	loop = g_main_loop_new (NULL, TRUE);
 
-	base_uri = soup_test_server_get_uri (server, "http", NULL);
-	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-
-	g_test_add_data_func ("/chunks/request/unstreamed", GINT_TO_POINTER (0), do_request_test);
-	g_test_add_data_func ("/chunks/request/proper-streaming", GINT_TO_POINTER (PROPER_STREAMING), do_request_test);
-	g_test_add_data_func ("/chunks/request/proper-streaming/restart", GINT_TO_POINTER (PROPER_STREAMING | RESTART), do_request_test);
-	g_test_add_data_func ("/chunks/request/hacky-streaming", GINT_TO_POINTER (HACKY_STREAMING), do_request_test);
-	g_test_add_data_func ("/chunks/request/hacky-streaming/restart", GINT_TO_POINTER (HACKY_STREAMING | RESTART), do_request_test);
-	g_test_add_func ("/chunks/response", do_response_test);
-	g_test_add_func ("/chunks/temporary", do_temporary_test);
-	g_test_add_func ("/chunks/large", do_large_chunk_test);
-
-	ret = g_test_run ();
-
-	soup_test_session_abort_unref (session);
-
+	base_uri = soup_uri_new ("http://127.0.0.1");
+	soup_uri_set_port (base_uri, port);
+	do_chunk_tests (base_uri);
 	soup_uri_free (base_uri);
 
 	g_main_loop_unref (loop);
 	soup_test_server_quit_unref (server);
 
 	test_cleanup ();
-	return ret;
+	return errors != 0;
 }
